/*
 * This is the entry point for the DKU Matrix Multiply demonstration application
 */

package Application;

import Helper.BAADParam;
import Helper.ParameterBag;
import dku.*;

/**
 * 
 * 
 * @author seanhalle@yahoo.com
 */
public class Main 
 { 
   static Main          application;
   static Scheduler     scheduler;
   
   //instance data..  will make an instance of Main class and call run() on it
   RowMajorMatrix       leftMatrix;
   ColMajorMatrix       rightMatrix;
   MatrixProductPiece   matrixProductRootPiece;
   
   ParameterBag         paramBag;
   String               leftMatrixFileName;
   String               rightMatrixFileName;
   String               resultMatrixFileName;
    /**
     * @param args the command line arguments
     */
   public static void main(String[] args) 
    {
      application = new Main( args );
      application.run();
    }
   
   public Main( String[] args )
    { String
      paramFileName = args[0];
      paramBag = new ParameterBag( paramFileName );
      BAADParam 
      temp = paramBag.getParam( "leftMatrixFileName" );
      leftMatrixFileName = temp.getStringValue();      
      temp = paramBag.getParam( "rightMatrixFileName" );
      rightMatrixFileName = temp.getStringValue();      
      temp = paramBag.getParam( "resultMatrixFileName" );
      resultMatrixFileName = temp.getStringValue();      
      
      int leftMatrixRows, leftMatrixCols, rightMatrixRows, rightMatrixCols;
      temp = paramBag.getParam( "leftMatrixRows" );
      leftMatrixRows = temp.getIntValue();
      temp = paramBag.getParam( "leftMatrixCols" );
      leftMatrixCols = temp.getIntValue();
      temp = paramBag.getParam( "rightMatrixRows" );
      rightMatrixRows = temp.getIntValue();
      temp = paramBag.getParam( "rightMatrixCols" );
      rightMatrixCols = temp.getIntValue();
      
      leftMatrix = new RowMajorMatrix( leftMatrixRows, leftMatrixCols );
      rightMatrix = new ColMajorMatrix( rightMatrixRows, rightMatrixCols );
      
      scheduler = new Scheduler();
    }
   
   long startTime, endTime;
   float timeToExecute;
   public void run()
    { // read data in from files then create the root piece      
      leftMatrix.fillSelfFromFile( leftMatrixFileName );
      rightMatrix.fillSelfFromFile( rightMatrixFileName );
      
      matrixProductRootPiece = new MatrixProductPiece( leftMatrix, rightMatrix);
      
      
      // perform calculations on data
      startTime = System.nanoTime();
      scheduler.processDataIn( matrixProductRootPiece );
      endTime = System.nanoTime();
      timeToExecute = ((float) (endTime - startTime)) / 1000000000;
      System.out.println( "timeToExecute: " + timeToExecute );
      System.exit(0);
    }
   
 }
