/*
 * BAADParam.java
 *
 */

package Helper;

/** This wraps parameters, so that different types of parameters can
 *   all be held in the same "bag" of parameters..
 *  A BAADParam has a name-field, which must be unique
 *
 * @author seanhalle@yahoo.com
 */
public class BAADParam
 {
   String      name;
   ParamType   type;
   
   int         intValue;
   double       doubleValue;
   Object      objValue;
   String      stringValue;
   
   /** Creates a new instance of BAADParam */
   public BAADParam()
    {
    }
   public BAADParam( String _name, int _intValue )
    { name     = _name;
      type     = ParamType.intType;
      intValue = _intValue;
    }
   public BAADParam( String _name, double _doubleValue)
    { name     = _name;
      type     = ParamType.doubleType;
      doubleValue = _doubleValue;
    }
   public BAADParam( String _name, String _strValue )
    { name     = _name;
      type     = ParamType.stringType;
      stringValue = _strValue;
    }
   public BAADParam( String _name, Object _objValue )
    { name     = _name;
      type     = ParamType.objType;
      objValue = _objValue;
    }
  

   public void setName( String _name )
    { name = _name;
    }
   public String getName()
    { return name;
    }   
   
  
   public void setType( ParamType _type )
    { type = _type;
    }
   public ParamType getType()
    { return type;
    }   
   
   
   public void setIntValue( int _intValue )
    { intValue = _intValue;
    }
   public int getIntValue()
    { return intValue;
    }   
   
   
   public void setDoubleValue( double _doubleValue )
    { doubleValue = _doubleValue;
    }
   public double getDoubleValue()
    { return doubleValue;
    }   
   
   
   public void setStringValue( String _stringValue )
    { stringValue = _stringValue;
    }
   public String getStringValue()
    { return stringValue;
    }   
   
   
   public void setObjValue( Object _objValue )
    { objValue = _objValue;
    }
   public Object getObjValue()
    { return objValue;
    }   
   
   
   
   public String toString()
    { switch( type )
       { case intType:     return String.valueOf( intValue );
         case doubleType:   return String.valueOf( doubleValue );
         case objType:     return objValue.toString();
         case stringType:  return stringValue;
       }
      return "";  // never reached, just makes editor happy
    }
 }
