/*
 * Debug.java
 *
 *
 */

package Helper;

import java.util.Iterator;
import java.util.List;

/**
 *
 * @author seanhalle@yahoo.com
 */
public class Debug
 {
   String      ancestorID = "";
   boolean     haveAncestorID = false;
   String      ownID;
   
   public static int   currHWCycle = 0;
   
   
   /**
    * These static status flags are used to turn on and off different
    *   groups of debugging statements..<p>
    *  Use via "d.msg( d.dep, ... );"<p>
    *  This form checks the flag in the first position to decide 
    *   whether it prints the debug message or not..<p>
    *  Add static flags of any convenient name, or change names of
    *   these to be meaningful..
    */
   public static final boolean   on  = true;
   public static final boolean   off = false;
   
      // set to false when want all 'if' statements to be 
      //  compiled-away as unreachable dead-code
   public static final boolean   debugIsEnabled = false;
   
      // by making these final, hopefully d.msg calls compiled away
   public static boolean   ss     = on;  // single step
   public static boolean   ss1    = on;  // verboser single st
   public static boolean   ss2    = on;  // verbosest single st
   public static boolean   pls    = off;  // 
   public static boolean   ltg    = off;  // load time gen trace
   public static boolean   st     = off;  // stats gather
   public static boolean   bkp    = off;  // breakpoints
   public static boolean   parse  = off;  // debug parsing of params
   public static final boolean pli= off;  // print loop instrs
   
   /** Creates a new instance of Debug */
   public Debug ( String _debugID )
    { ownID = _debugID;
    }
   public Debug ( String _ancestorID, String _ownID )
    { ancestorID = _ancestorID;
      haveAncestorID = true;
      ownID = _ownID;
    }
   

   //===================================================
   
   // Printing
   public void msg( String _msg )
    { 
      if( haveAncestorID )
       { msg( currHWCycle, ancestorID + ":: " + ownID + ":: " + _msg);
       }
      else
         msg( currHWCycle, ownID + ":: " + _msg );
    }
   public void msg( int line, String _msg )
    { System.out.println( line + " | " + _msg );
    }
   
   public void msg( String _msg, BAADInstr instr )
    { 
      if( instr.isCorrectlyPred )
       { msg( _msg + " I: " + instr.getDynamicPos() +
              " op: " + instr.getInstrFuncType() );
       }
      else
       { msg( _msg + " I: " + instr.getDynamicPos() +
              " op: " + instr.getInstrFuncType() + "   mp" );
       }
    }
   public void msg( String _msg, BAADInstr instr1, BAADInstr instr2 )
    {
      if( instr1.isCorrectlyPred )
       { _msg += " I: " + instr1.getDynamicPos() +
              " op: " + instr1.getInstrFuncType();
       }
      else
       { _msg += " I: " + instr1.getDynamicPos() +
              " op: " + instr1.getInstrFuncType() + "  mp";
       }
      
      if( instr2.isCorrectlyPred )
       { msg( _msg + " | I: " + instr2.getDynamicPos() +
              " op: " + instr2.getInstrFuncType() );
       }
      else
       { msg( _msg + " | I: " + instr2.getDynamicPos() +
              " op: " + instr2.getInstrFuncType() + "   mp" );
       }
    }
   public void msg( String _msg, int num )
    { 
      msg( _msg + ": " + num );
    }
   public void msg( String _msg, boolean bool )
    {
      msg( _msg + ": " + bool );
    }
   public void msg( String _msg, int num, boolean bool )
    {
      msg( _msg + ": " + num + " : " + bool );
    }
   public void msg( String _msg, int num1, BAADInstr instr )
    { 
      msg( _msg + ": " + num1, instr );
    }
   public void msg( String _msg, int num1, int num2 )
    { 
      msg( _msg + ": " + num1 + " : " + num2 );
    }
   public void msg( String _msg, int num1, int num2, BAADInstr instr )
    { 
      msg( _msg + ": " + num1 + " : " + num2, instr );
    }   
   
   
   //============================================
   
   
   /**
    * use the static status flags as the first parameter.. this
    *   allows having separate groups of debug statements which
    *   can all be turned on and off together.<p>
    *  Usage:  "d.msg( d.dep, ... );"<p>
    *  When want debugging to go away, set "debugIsEnabled", up at the
    *   top of this Class, to false.  This makes all the d.msg 
    *   statements un-reachable, so compiler should eliminate them
    *   as dead-code and not even perform the "if" statement, 
    *   hopefully just removing the entire method call.<p>
    */
   public void msg( boolean isOn, String _msg)
    { if( debugIsEnabled && isOn ) msg( _msg );
    }
      
   public void msg( boolean isOn, String _msg, BAADInstr instr )
    { if( debugIsEnabled && isOn ) msg( _msg, instr );
    }
   
   public void msg( boolean isOn, String _msg, BAADInstr instr1,
       BAADInstr instr2 )
    { if( debugIsEnabled && isOn ) msg( _msg, instr1, instr2 );
    }
   
   public void msg( boolean isOn, String _msg, int num )
    { if( debugIsEnabled && isOn ) msg( _msg, num );
    }
   
   public void msg( boolean isOn, String _msg, boolean bool )
    { if( debugIsEnabled && isOn ) msg( _msg, bool );
    }
   
   public void msg( boolean isOn, String _msg, int num, boolean bool )
    { if( debugIsEnabled && isOn ) msg( _msg, num, bool );
    }
   
   public void msg( boolean isOn, String _msg, int num, 
    BAADInstr instr)
    { if( debugIsEnabled && isOn ) msg( _msg, num, instr );
    }
   
   public void msg( boolean isOn, String _msg, int num, int num2, 
    BAADInstr instr )
    { if( debugIsEnabled && isOn ) msg( _msg, num, num2, instr );
    }
   
   //============================================
   public void msg( boolean isOn, String _msg, double[] instr )
    { if( debugIsEnabled && isOn ) msg( _msg + instrToString(instr) );
    }
   public void msg( boolean isOn, String _msg, double val )
    { if( debugIsEnabled && isOn ) msg( _msg + val );
    }
   
   public String instrToString( double[] instr )
    { String temp = "";
      temp += instr[0];
      temp += " | " + instr[1];
      temp += " | " + instr[2];
      temp += " | " + instr[3];
      temp += " | " + instr[4];
      temp += " | " + instr[5];
      temp += " | " + instr[6];
      return temp;
    }
   
   //============================================
   
   
   public void msg( List<BAADInstr> list )
    { Iterator<BAADInstr>
       iter = list.iterator();
      BAADInstr instr;
      while( iter.hasNext() )
       { instr = iter.next();
         System.out.print( "  I: " + instr.getDynamicPos () );
       }
      System.out.print("\n");
    }
   
   public void errMsg( String _msg )
    { System.err.println( "ERROR: " + _msg );
    }
   
   public void errMsg( String _msg, BAADInstr instr )
    { errMsg( _msg + " I: " + instr.getDynamicPos() +
              " op: " + instr.getInstrFuncType() );      
    }
 }
