/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package dku;

/**
 * the static method "processDataIn" is called 
 * @author Me
 */
public class Scheduler 
 {
   /**
    * This method is called when one desires to initiate processing data.
    *  The data is placed into some object that implements the DKUPiece
    *  interface, then this method is statically called.
    * The single call "Scheduler.processDataIn( rootPieceOfData );" causes a
    *  whole flurry of activity. 
    * First this method invokes the divider, to create
    *  a stream of sub-pieces.  Each sub-piece in the returned stream is either
    *  further divided (by a pseudo-recursive call from inside processDataIn)
    *  or it is given to its own copy of the kernel method.
    * When the kernel is done, the finished sub-piece is sent to the undivide
    *  method of its parent.
    * 
    * When this method returns, all processing on the data in the piece is
    *  complete.  The result of processing appears as a side-effect.  If the
    *  processing result is inside some new data-structure that was
    *  created, then the DKUPiece implementation must include a pointer to
    *  the result data-structure.  Then when the call to processDataIn returns,
    *  the pointer within the root DKUPiece will be valid and point to the 
    *  result data-structure.
    * 
    * The implementation given here is not necessarily what runs.  This is the
    *  serial default.  When this program is passed through
    *  the back-end compiler or pre-processor, the tool will decide whether
    *  the parallelism can be made of appropriate granularity for the hardware.
    * All calls to this method that can be made of appropriate granularity
    *  will be replaced by something hardware-specific.  It could be a call to
    *  an ExeMoI (execution model implementation == run-time system), which
    *  will dynamically decide how many sub-pieces to divide the piece into, or
    *  the calls to this method might be replaced by a static scheduling of
    *  the divider, plus static messages to send the sub-pieces to processing
    *  resources.
    * 
    * 
    *  When transform this DKU pattern into the eight threaded version for
    *   octopus, will make eight threads inside the scheduler, and call the
    *   kernel of a different MatrixProductPiece inside each of the threads,
    *   in round-robin fashion.
    *  What's important is that the interface of all methods in this scheduler
    *   match the interface of the 8 threaded version..  that allows the
    *   8 threaded scheduler to be written independently of this application.
    *   The 8thd scheduler can be swapped in to any application that has this
    *   same interface.
    *  That ability to swap in a different scheduler implementation blindly
    *   into any application is what gives the portability property of DKU.
    */
   public void processDataIn( DKUPiece rootPiece  )
    { 
      /* This commented code looks something like what a scheduler loop might
       *  be implemented as, if it only did one level of division.
      rootPiece.divideSelfInto_SubPieces( 0 );
      while( rootPiece.hasMoreSubPieces() )  // doesn't need to be sequential
       { nextSubPieceToCalc = rootPiece.giveNextSubPiece();
         nextSubPieceToCalc.performKernelOnSelf();
         rootPiece.unDivideASubPiece( nextSubPieceToCalc );
       }
       * 
       * Here, however, this implementation of the scheduler is the 
       *  un translated serial version.
       * Know a-priori that there are no sub-pieces.  That makes the passed-in
       *  piece a leaf piece.  A leaf piece is an AWU, which is scheduled
       *  directly onto a processor, meaning the kernel gets invoked on leaf
       *  pieces (and only on leaf pieces).  So invoke the kernel on the
       *  passed-in root piece.
       */
      
      rootPiece.performKernelOnSelf();
    }
 }
