/*
 * ParameterBag.java
 *
 * Created on August 10, 2005, 12:27 AM
 *
 */

package Helper;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Scanner;

/** This one "bag" of architectural parameters is passed to everything
 *   that needs to know parameter values..  a single bag is created,
 *   all the parameters are placed into it as name-value pairs, then
 *   it is passed to whatever needs param values.
 *
 * @author seanhalle@yahoo.com
 */
public class ParameterBag
 {
   HashMap<String, BAADParam> internalHashMap;
   
   Debug d;
   
   /** Creates a new instance of ParameterBag */
   public ParameterBag ( String _paramFileName )
    { internalHashMap = new HashMap<String, BAADParam>();
      d = new Debug( "ParameterBag" );
      parseParameterBag( _paramFileName );
    }
   
   public void putParam( BAADParam paramObj )
    { if ( this.getParam( paramObj.getName() ) != null )
       { System.err.println( "\nSetting same param name twice: " +
          paramObj.getName() );
       }
      internalHashMap.put( paramObj.getName(), paramObj );
    }
   public BAADParam getParam( String paramName )
    { return internalHashMap.get( paramName );
    }  
   
   public Iterator<BAADParam> giveIter()
    { return internalHashMap.values().iterator();
    }
   
      
   private void parseParameterBag( String _paramFileName )
    { File paramFile;
      Scanner paramScanner;
      String lineStart;
            
      String paramFileName = _paramFileName;
      try
       { paramFile    = new File( paramFileName );
         paramScanner = new Scanner( paramFile );
       }
      catch( Exception e )
       { paramFile    = null;
         paramScanner = null;
         System.err.println( "couldn't open file: " + paramFileName );
       }
            
      paramScanner.useDelimiter(",\\s*|\n|\r\n");
      
      int    intValue;
      float  floatValue;
      String stringValue, key, objClassName, paramString;
      Object objValue;
      while( paramScanner.hasNext() )
       { lineStart = paramScanner.next();
         if( lineStart.equals("int") )
          { key = paramScanner.next();
            stringValue = paramScanner.next();
            intValue = Integer.parseInt( stringValue );
            this.putParam( new BAADParam( key, intValue ) );
                  d.msg(d.parse, "parsed " + key, intValue );
          }
         else if( lineStart.equals("float") )
          { key = paramScanner.next();
            floatValue = paramScanner.nextFloat();
            this.putParam( new BAADParam( key, floatValue ) );
                  d.msg(d.parse, "parsed " + key + floatValue );
          }
         else if( lineStart.equals("double") )
          { key = paramScanner.next();
            floatValue = paramScanner.nextFloat();
            this.putParam( new BAADParam( key, floatValue ) );
                  d.msg(d.parse, "parsed " + key + floatValue );
          }
         else if( lineStart.equals("string") )
          { key = paramScanner.next();
            stringValue = paramScanner.next();
            this.putParam( new BAADParam( key, stringValue ));
                  d.msg(d.parse, "parsed "+ key + " : "+ stringValue);
          }
         else if( lineStart.equals("objStart") )
          { key = paramScanner.next();
            objClassName = paramScanner.next();
            paramString = "";
            paramScanner.nextLine();  // get rid of empty end-of-line
            while( paramScanner.hasNextLine() &&
                   ! paramScanner.hasNext("objEnd") )
             { paramString += paramScanner.nextLine() + "\n";
             }
            Class theClass = null;
            objValue = null;
             
               // <snip>  ignore obj values for loop solver
            
            paramScanner.nextLine();  // get rid of "objEnd" line
          }
       }
    }

 }
